@@ -37,6 +37,7 @@
 #include <sys/time.h>
 #include <sys/utsname.h>
 #include <time.h>
+#include <locale.h>
 
 /* Legal conversion specifiers, as specified in the C standard. */
 #define C_STANDARD_STRFTIME_CHARACTERS "aAbBcdHIjmMpSUwWxXyYZ"
@@ -53,72 +54,6 @@ typedef struct {
 
 static GList *hash_tables_to_free_at_exit;
 
-/* We will need this for eel_unsetenv if there is no unsetenv. */
-#if !defined (HAVE_SETENV)
-extern char **environ;
-#endif
-
-/**
- * eel_setenv:
- * 
- * Adds "*name=*value" to the environment
- *
- * Returns: see "putenv" - 
- * 
- **/
-int
-eel_setenv (const char *name, const char *value, gboolean overwrite)
-{
-#if defined (HAVE_SETENV)
-	return setenv (name, value, overwrite);
-#else
-	char *string;
-	
-	if (! overwrite && g_getenv (name) != NULL) {
-		return 0;
-	}
-	
-	/* This results in a leak when you overwrite existing
-	 * settings. It would be fairly easy to fix this by keeping
-	 * our own parallel array or hash table.
-	 */
-	string = g_strconcat (name, "=", value, NULL);
-	return putenv (string);
-#endif
-}
-
-/**
- * eel_unsetenv:
- * 
- * Removes *name from the environment
- * 
- **/
-void
-eel_unsetenv (const char *name)
-{
-#if defined (HAVE_SETENV)
-	unsetenv (name);
-#else
-	int i, len;
-
-	len = strlen (name);
-	
-	/* Mess directly with the environ array.
-	 * This seems to be the only portable way to do this.
-	 */
-	for (i = 0; environ[i] != NULL; i++) {
-		if (strncmp (environ[i], name, len) == 0
-		    && environ[i][len] == '=') {
-			break;
-		}
-	}
-	while (environ[i] != NULL) {
-		environ[i] = environ[i + 1];
-		i++;
-	}
-#endif
-}
-
 /**
  * eel_g_date_new_tm:
  * 
@@ -401,7 +336,7 @@ eel_g_str_list_copy (GList *list)
 GList *
 eel_g_str_list_alphabetize (GList *list)
 {
-	return g_list_sort (list, (GCompareFunc) eel_strcoll);
+	return g_list_sort (list, (GCompareFunc) g_utf8_collate);
 }
 
 int
@@ -776,69 +711,6 @@ eel_g_slist_from_g_list (GList *list)
 	return g_slist_reverse (slist);
 }
 
-/**
- * eel_dumb_down_for_multi_byte_locale_hack
- * 
- * Return value: A boolean value indicating whether the current locale
- *               is multi byte so that we can dumb down some operations.
- *               to work on those locales.  This is a temporary workaround
- *               with will be properly fixed in a future version of 
- *               eel.
- *
- */
-gboolean
-eel_dumb_down_for_multi_byte_locale_hack (void)
-{
-	static gboolean is_multi_byte_locale = FALSE;
-	static gboolean is_multi_byte_locale_known = FALSE;
-	guint i;
-	const char *variable = NULL;
-
-	/*
-	 * List of environment variables that effect the locale.
-	 * This list was provided by John Harper (out of Sawfish)
-	 * where he uses it for similar purposes.
-	 */
-	static const char *locale_variables[] ={
-		"LANGUAGE",
-		"LC_ALL",
-		"LC_MESSAGES",
-		"LANG",
-		"GDM_LANG"
-	};
-
-	/*
-	 * List of locales (prefixes) known to be multi byte.
-	 */
-	static const char *multi_byte_locales[] ={
-		"ja",
-		"ko",
-		"zh"
-	};
-
-	/* Find out if the locale is multi byte only once */
-	if (is_multi_byte_locale_known) {
-		return is_multi_byte_locale;
-	}
-	is_multi_byte_locale_known = TRUE;
-
-	/* Find the first language variable that is set */
-	for (i = 0; i < G_N_ELEMENTS (locale_variables) && variable == NULL; i++) {
-		variable = g_getenv (locale_variables[i]);
-	}
-
-	/* If a language variable was found, check it agains the known multi byte locales */
-	if (variable != NULL) {
-		for (i = 0; i < G_N_ELEMENTS (multi_byte_locales); i++) {
-			if (eel_istr_has_prefix (variable, multi_byte_locales[i])) {
-				is_multi_byte_locale = TRUE;
-			}
-		}
-	}
-
-	return is_multi_byte_locale;
-}
-
 /* Return the operating system name: Linux, Solaris, etc. */
 char *
 eel_get_operating_system_name (void)
@@ -983,6 +855,103 @@ eel_remove_weak_pointer (gpointer pointer_location)
 	*object_location = NULL;
 }
 
+/* Get the filename encoding, returns TRUE if utf8 */
+
+typedef struct _EelFilenameCharsetCache EelFilenameCharsetCache;
+
+struct _EelFilenameCharsetCache {
+  gboolean is_utf8;
+  gchar *charset;
+  gchar *filename_charset;
+};
+
+static void
+filename_charset_cache_free (gpointer data)
+{
+  EelFilenameCharsetCache *cache = data;
+  g_free (cache->charset);
+  g_free (cache->filename_charset);
+  g_free (cache);
+}
+
+/*
+ * eel_get_filename_charset:
+ * @charset: return location for the name of the filename encoding 
+ *
+ * Determines the character set used for filenames by consulting the 
+ * environment variables G_FILENAME_ENCODING and G_BROKEN_FILENAMES. 
+ *
+ * G_FILENAME_ENCODING may be set to a comma-separated list of character 
+ * set names. The special token "@locale" is taken to mean the character set 
+ * for the current locale. The first character set from the list is taken 
+ * as the filename encoding. 
+ * If G_FILENAME_ENCODING is not set, but G_BROKEN_FILENAMES is, the
+ * character set of the current locale is taken as the filename encoding.
+ *
+ * The returned @charset belongs to Eel and must not be freed.
+ * 
+ * Return value: %TRUE if the charset used for filename is UTF-8.
+ */
+gboolean
+eel_get_filename_charset (const gchar **filename_charset)
+{
+  static GStaticPrivate cache_private = G_STATIC_PRIVATE_INIT;
+  EelFilenameCharsetCache *cache = g_static_private_get (&cache_private);
+  const gchar *charset;
+  
+  if (!cache)
+    {
+      cache = g_new0 (EelFilenameCharsetCache, 1);
+      g_static_private_set (&cache_private, cache, filename_charset_cache_free);
+    }
+
+  g_get_charset (&charset);
+
+  if (!(cache->charset && strcmp (cache->charset, charset) == 0))
+    {
+      const gchar *new_charset;
+      gchar *p, *q;
+
+      g_free (cache->charset);
+      g_free (cache->filename_charset);
+      cache->charset = g_strdup (charset);
+      
+      p = getenv ("G_FILENAME_ENCODING");
+      if (p != NULL) 
+	{
+	  q = strchr (p, ',');
+	  if (!q) 
+	    q = p + strlen (p);
+
+	  if (strncmp ("@locale", p, q - p) == 0)
+	    {
+	      cache->is_utf8 = g_get_charset (&new_charset);
+	      cache->filename_charset = g_strdup (new_charset);
+	    }
+	  else
+	    {
+	      cache->filename_charset = g_strndup (p, q - p);
+	      cache->is_utf8 = (strcmp (cache->filename_charset, "UTF-8") == 0);
+	    }
+	}
+      else if (getenv ("G_BROKEN_FILENAMES") != NULL)
+	{
+	  cache->is_utf8 = g_get_charset (&new_charset);
+	  cache->filename_charset = g_strdup (new_charset);
+	}
+      else 
+	{
+	  cache->filename_charset = g_strdup ("UTF-8");
+	  cache->is_utf8 = TRUE;
+	}
+    }
+
+  if (filename_charset)
+    *filename_charset = cache->filename_charset;
+
+  return cache->is_utf8;
+}
+
 #if !defined (EEL_OMIT_SELF_CHECK)
 
 static void 
@@ -1151,19 +1120,17 @@ eel_self_check_glib_extensions (void)
 	memset (huge_string, 'a', 10000);
 	huge_string[10000] = '\0';
 
+	setlocale (LC_TIME, "C");
+
 	EEL_CHECK_STRING_RESULT (test_strftime ("", 2000, 1, 1, 0, 0, 0), "");
 	EEL_CHECK_STRING_RESULT (test_strftime (huge_string, 2000, 1, 1, 0, 0, 0), huge_string);
 	EEL_CHECK_STRING_RESULT (test_strftime ("%%", 2000, 1, 1, 1, 0, 0), "%");
 	EEL_CHECK_STRING_RESULT (test_strftime ("%%%%", 2000, 1, 1, 1, 0, 0), "%%");
-	/* localizers: These strings are part of the strftime
-	 * self-check code and must be changed to match what strtfime
-	 * yields. The first one is "%m/%d/%y, %I:%M %p".
-	 */
-	EEL_CHECK_STRING_RESULT (test_strftime ("%m/%d/%y, %I:%M %p", 2000, 1, 1, 1, 0, 0), _("01/01/00, 01:00 AM"));
-	/* The second one is "%-m/%-d/%-y, %-I:%M %p". */
-	EEL_CHECK_STRING_RESULT (test_strftime ("%-m/%-d/%y, %-I:%M %p", 2000, 1, 1, 1, 0, 0), _("1/1/00, 1:00 AM"));
-	/* The third one is "%_m/%_d/%_y, %_I:%M %p". */
-	EEL_CHECK_STRING_RESULT (test_strftime ("%_m/%_d/%y, %_I:%M %p", 2000, 1, 1, 1, 0, 0), _(" 1/ 1/00,  1:00 AM"));
+	EEL_CHECK_STRING_RESULT (test_strftime ("%m/%d/%y, %I:%M %p", 2000, 1, 1, 1, 0, 0), "01/01/00, 01:00 AM");
+	EEL_CHECK_STRING_RESULT (test_strftime ("%-m/%-d/%y, %-I:%M %p", 2000, 1, 1, 1, 0, 0), "1/1/00, 1:00 AM");
+	EEL_CHECK_STRING_RESULT (test_strftime ("%_m/%_d/%y, %_I:%M %p", 2000, 1, 1, 1, 0, 0), " 1/ 1/00,  1:00 AM");
+
+	setlocale (LC_TIME, "");
 
 	g_free (huge_string);
 
