@@ -28,6 +28,7 @@
 #include <errno.h>
 #include <locale.h>
 #include <stdlib.h>
+#include <string.h>
 
 #if !defined (EEL_OMIT_SELF_CHECK)
 #include "eel-lib-self-check-functions.h"
@@ -577,25 +578,25 @@ skip_argv (va_list *va,
 		return;
 		
 	case ARG_TYPE_INT:
-		va_arg (*va, int);
+		(void) va_arg (*va, int);
 		break;
 	case ARG_TYPE_LONG:
-		va_arg (*va, long int);
+		(void) va_arg (*va, long int);
 		break;
 	case ARG_TYPE_LONG_LONG:
-		va_arg (*va, long long int);
+		(void) va_arg (*va, long long int);
 		break;
 	case ARG_TYPE_SIZE:
-		va_arg (*va, gsize);
+		(void) va_arg (*va, gsize);
 		break;
 	case ARG_TYPE_LONG_DOUBLE:
-		va_arg (*va, long double);
+		(void) va_arg (*va, long double);
 		break;
 	case ARG_TYPE_DOUBLE:
-		va_arg (*va, double);
+		(void) va_arg (*va, double);
 		break;
 	case ARG_TYPE_POINTER:
-		va_arg (*va, void *);
+		(void) va_arg (*va, void *);
 		break;
 	}
 }
@@ -745,7 +746,7 @@ eel_strdup_vprintf_with_custom (EelPrintfHandler *custom,
 		if (conversions[i].precision_pos != -1) {
 			char *val;
 			
-			va_copy(va, va_orig);
+			G_VA_COPY(va, va_orig);
 			skip_to_arg (&va, args, custom, conversions[i].precision_pos);
 			val = g_strdup_vprintf ("%d", va);
 			va_end (va);
@@ -760,7 +761,7 @@ eel_strdup_vprintf_with_custom (EelPrintfHandler *custom,
 		if (conversions[i].width_pos != -1) {
 			char *val;
 			
-			va_copy(va, va_orig);
+			G_VA_COPY(va, va_orig);
 			skip_to_arg (&va, args, custom, conversions[i].width_pos);
 			val = g_strdup_vprintf ("%d", va);
 			va_end (va);
@@ -772,7 +773,7 @@ eel_strdup_vprintf_with_custom (EelPrintfHandler *custom,
 			g_free (val);
 		}
 		
-		va_copy(va, va_orig);
+		G_VA_COPY(va, va_orig);
 		skip_to_arg (&va, args, custom, conversions[i].arg_pos);
 		type = args[conversions[i].arg_pos];
 		if (type < 0) {
@@ -882,25 +883,26 @@ void
 eel_ref_str_unref (eel_ref_str str)
 {
 	volatile gint *count;
-	gint res;
+	gint old_ref;
 
 	if (str == NULL)
 		return;
 	
 	count = (volatile gint *)((char *)str - sizeof (gint));
 
-	res = g_atomic_int_exchange_and_add (count, -1);
-
-	if (res == 0) {
+	old_ref = g_atomic_int_get (count);
+	if (old_ref == 1) {
 		g_free ((char *)count);
-	} else if (res == 0x80000000) {
+	} else if (old_ref == 0x80000001) {
 		G_LOCK (unique_ref_strs);
-		/* Need to recheck after taking lock to avoid lookup in _get_unique() */
-		if (g_atomic_int_get (count) == 0x80000000) {
+		/* Need to recheck after taking lock to avoid races with _get_unique() */
+		if (g_atomic_int_exchange_and_add (count, -1) == 0x80000001) {
 			g_hash_table_remove (unique_ref_strs, (char *)str);
-		}
+			g_free ((char *)count);
+		} 
 		G_UNLOCK (unique_ref_strs);
-		g_free ((char *)count);
+	} else {
+		g_atomic_int_exchange_and_add (count, -1);
 	}
 }
 
@@ -949,7 +951,7 @@ custom1_to_string (char *format, va_list va)
 static void
 custom1_skip (va_list *va)
 {
-	va_arg (*va, int);
+	(void) va_arg (*va, int);
 }
 
 static char *
@@ -965,7 +967,7 @@ custom2_to_string (char *format, va_list va)
 static void
 custom2_skip (va_list *va)
 {
-	va_arg (*va, char *);
+	(void) va_arg (*va, char *);
 }
 
 static EelPrintfHandler handlers[] = {
